"""Common configuration constants
"""

PROJECTNAME = 'collective.tablepage'

ADD_PERMISSIONS = {
    'TablePage': 'collective.tablepage: Add TablePage',
}

MANAGE_TABLE = "collective.tablepage: Manage Table"